//+------------------------------------------------------------------+
//|                                                    Swing 1_2.mq4 |
//|                                                        kingkevbo |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "kingkevbo"
#property link      ""

#define BUYING 1
#define SELLING 2

// global variables settable by user
extern int TimeFrame = 30;
extern double OrderOneTakeProfit = 100;
extern double TakeProfitInterval = 50;
extern int NumberOrders = 10;
extern bool TrailingOrder = false;
extern double TrailingStop = 200;
extern double StopLoss = 200;
extern double RiskPercent = 2;
extern bool UseMM = false;

// global variables
int SWING = 1;
int cSWING = 0;
double currentValue;
double oldBuyPrice, buyPrice;
double oldSellPrice, sellPrice;
datetime oldTime;
bool firstRun = true;
double risk;
int MagicNumber = 12345;
int winFileHandle, lossFileHandle;

// arrays to store order tickets
int myBuyTickets[11];
int mySellTickets[11];

//array to store balances for money management
double balances[3];


//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
  if( firstRun == true ) // on the first run only, (init is called under various curcumstances during program run)
  {
    firstRun = false;
    
    winFileHandle = FileOpen("wins.csv",FILE_CSV|FILE_WRITE,',');
    lossFileHandle = FileOpen("losses.csv",FILE_CSV|FILE_WRITE,',');
    
    if( UseMM == true ) // are we using money management?
    {
      // yes, store opening account balance in balances array
      balances[0] = AccountBalance();
      // set risk as if we have won
      MoneyManagement(1);
      
    }
  }
  return(0);
}
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
{
  return(0);
}
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{
  // check the number of orders is not too big, or this will
  // overrun our arrays
  if( NumberOrders > 10 )
  {
    Comment("Number orders too big (>10)");
    return(0);
  }
  
  // run swing state machine
  Swing();
  
  return(0);
}
//+------------------------------------------------------------------+

void Swing()
{

  static int direction;
  int i;
  int ordersClosed;
  
  if( SWING == 0 )
  {
    // task is disabled
    cSWING = 0;
    return;
  }
  
  switch(cSWING)
  {
    case 0:
      
      // init variables
      direction = 0;
      cSWING++;
      break;
      
    case 1:
    
      // check swing zz values
      if( oldTime < iTime(Symbol(),TimeFrame,0) )
      {
        // only do this on a new 15 min candle as this is
        // the only time it'll change anyway, or this program will run
        // veeeerrrry slooowwwly.
        oldTime = iTime(Symbol(),TimeFrame,0);
    
        if( CheckSwingValues() == true )
        {
          // swing values have changed
          Print("Swing values changed, deleting old orders and placing new ones");
          DeleteOldOrders();
          PlaceStopOrders();
        }
      }
      
      // check to see if we have an order open;
      if( myBuyTickets[0] > 0 )
      {
        OrderSelect(myBuyTickets[0],SELECT_BY_TICKET);
        if( OrderType() == OP_BUY && OrderCloseTime() == 0 )
        {
          // order has been opened, and its not an old closed order
          Print("Buy order opened...");
          direction = BUYING;
          cSWING++;
          break; //<- END STEP HERE
        }
      }
      if( mySellTickets[0] > 0 )
      {
        OrderSelect(mySellTickets[0],SELECT_BY_TICKET);
        if( OrderType() == OP_SELL && OrderCloseTime() == 0 )
        {
          // order has been opened, and its not an old closed order
          Print("Sell order opened...");
          direction = SELLING;
          cSWING++;
          break; //<- END STEP HERE
        }
      }
      break;
      
    case 2:
    
      // now we wait here for the order to Close
      if( direction == BUYING )
      {
        OrderSelect(myBuyTickets[0],SELECT_BY_TICKET);
        if( OrderCloseTime() != 0 )
        {
          // order has closed, did it close at profit?
          if( OrderProfit() > 0 )
          {
            // order closed at profit, move other orders SL to BE
            Print("Buy Order 1 closed at profit, move SLs to BE");
            
            // if we are using MM, record a win.
            if( UseMM == true ) MoneyManagement(1);
            WriteDayAndTime(winFileHandle);
            
            for( i = 1 ; i < NumberOrders ; i++ )
            {
              OrderSelect(myBuyTickets[i],SELECT_BY_TICKET);
              OrderModify(myBuyTickets[i],OrderOpenPrice(),OrderOpenPrice(),OrderTakeProfit(),0);
            }
            if( TrailingOrder == true )
            {
              Print("Modifying trailing order");
              OrderSelect(myBuyTickets[NumberOrders],SELECT_BY_TICKET);
              OrderModify(myBuyTickets[NumberOrders],OrderOpenPrice(),OrderOpenPrice(),OrderTakeProfit(),0);
            }
            cSWING++;
            break; //<- END STEP HERE
          }
          else
          {
            // order closed at a loss
            Print("Buy order closed at a loss");
            // if we are using MM, record a loss.
            if( UseMM == true ) MoneyManagement(-1);
            WriteDayAndTime(lossFileHandle);
            cSWING++;
            break; //<- END STEP HERE
          }
        }
      }
      if( direction == SELLING )
      {
        OrderSelect(mySellTickets[0],SELECT_BY_TICKET);
        if( OrderCloseTime() != 0 )
        {
          // order has closed, did it close at profit?
          if( OrderProfit() > 0 )
          {
            // order closed at profit, move order 2 SL to BE
            Print("Sell Order 1 closed at profit, move SLs to BE");
            
            // if we are using MM, record a win.
            if( UseMM == true ) MoneyManagement(1);
            WriteDayAndTime(winFileHandle);
            
            for( i = 1 ; i < NumberOrders ; i++ )
            {
              OrderSelect(mySellTickets[i],SELECT_BY_TICKET);
              OrderModify(mySellTickets[i],OrderOpenPrice(),OrderOpenPrice(),OrderTakeProfit(),0);
            }
            if( TrailingOrder == true )
            {
              Print("Modifying trailing order");
              OrderSelect(mySellTickets[NumberOrders],SELECT_BY_TICKET);
              OrderModify(mySellTickets[NumberOrders],OrderOpenPrice(),OrderOpenPrice(),OrderTakeProfit(),0);
            }
            cSWING++;
            break; //<- END STEP HERE
          }
          else
          {
            // order closed at a loss
            Print("Sell order closed at a loss");
            // if we are using MM, record a loss.
            if( UseMM == true ) MoneyManagement(-1);
            WriteDayAndTime(lossFileHandle);
            cSWING++;
            break; //<- END STEP HERE
          }
        }
      }     
      break;
      
    case 3:
    
      // while orders are open, trail SL on trailing order, it it exists.
      if( TrailingOrder == true )
      {
        if( myBuyTickets[NumberOrders] > 0 )
        {
          if( direction == BUYING )
          {
            CheckTradeForStopLoss(myBuyTickets[NumberOrders], TrailingStop);
          }
        }
        if( mySellTickets[NumberOrders] > 0 )
        {
          if( direction == SELLING )
          {
            CheckTradeForStopLoss(mySellTickets[NumberOrders], TrailingStop);
          }
        }         
      }
      // wait here for orders to close
      if( direction == BUYING )
      {
        for( i = 0 ; i < NumberOrders ; i++ )
        {
          OrderSelect(myBuyTickets[i],SELECT_BY_TICKET);
          if( OrderCloseTime() != 0 )
          {
            ordersClosed++;
          }
        }
        if( ordersClosed == NumberOrders )
        {
          // all orders closed, unless training order is still open,
          // which we will deal with in the next step
          Print("All buy orders closed");
          cSWING++;
          break; //<- END STEP HERE
        }        
      }
      if( direction == SELLING )
      {
        for( i = 0 ; i < NumberOrders ; i++ )
        {
          OrderSelect(mySellTickets[i],SELECT_BY_TICKET);
          if( OrderCloseTime() != 0 )
          {
            ordersClosed++;
          }
        }
        if( ordersClosed == NumberOrders )
        {
          // all orders closed, unless training order is still open,
          // which we will deal with in the next step
          Print("All sell orders closed");
          cSWING++;
          break; //<- END STEP HERE
        }
      }
      break;
      
    case 4:
    
      // check to see if trailing order has closed, if it exists
      if( TrailingOrder == true )
      {
        if( direction == BUYING )
        {
          OrderSelect(myBuyTickets[NumberOrders],SELECT_BY_TICKET);
          if( OrderCloseTime() != 0 )
          {
            Print("Trailing order closed");
            cSWING = 0;
            break; //<- END STEP HERE
          }
        }
        if( direction == SELLING )
        {
          OrderSelect(mySellTickets[NumberOrders],SELECT_BY_TICKET);
          if( OrderCloseTime() != 0 )
          {
            Print("Trailing order closed");
            cSWING = 0;
            break; //<- END STEP HERE
          }
        }
        
        // if its still open, trail the stop.      
        if( myBuyTickets[NumberOrders] > 0 )
        {
          if( direction == BUYING )
          {
            CheckTradeForStopLoss(myBuyTickets[NumberOrders], TrailingStop);
          }
        }
        if( mySellTickets[NumberOrders] > 0 )
        {
          if( direction == SELLING )
          {
            CheckTradeForStopLoss(mySellTickets[NumberOrders], TrailingStop);
          }
        }         
      }
      else // there is no trailing order, back to the start.
      {
        cSWING = 0;
        break;
      }
      break;
      
      
  }
}

// The money management fuction will adjust risk depending on whether
// the last trade won or lost. This is an attempt to reduce losses
// during bad times for the system. For each win the function is run
// with 1 as an argument, and for each loss it is run with -1 as an argument.
// The fuction halves risk every time a trade looses, down to a level of 
// max risk/4. the prevoius higher level of risk is not returned to
// until the balance has recovered to what it was before the loss.
void MoneyManagement(int result)
{
  int i;
  static int count; // count stores the number of consecutive losses.
  
  // balances[0] stores the highest balance that there
  // has ever been. If this gets updated, the previous
  // lower balance levels are cleared and risk is set 
  // to max level.
  if( AccountBalance() >= balances[0] )
  {
    // balance is now at its highest level
    count = 0; // reset losses counter
    balances[0] = AccountBalance();
    
    // reset lower balance levels
    for( i = 1 ; i < 3 ; i++ )
    {
      balances[i] = 0.0;
    }
    risk = RiskPercent; // set risk to max level
    Print("Next trade risk will be at ", risk,"%");
    return;
  }
  
  if( result == -1 )
  {
    // record a loss
    count++;
    if( count < 3 )
    {
      // we only store the lowest balance after 2 consecutive losses,
      // as we only halve risk twicw
      balances[count] = AccountBalance();
    }
    switch(count)
    {
      case 1: risk = RiskPercent/2; break;
      case 2: risk = RiskPercent/4; break;
      default: risk = RiskPercent/4; break;
    }
    Print("Next trade risk will be at ", risk,"%");
    return;
  }
  
  if( result == 1 )
  {
    //record a win
    for( i = 2 ; i >= 0 ; i-- )
    {
      // check to see if current balance has increased enough
      // to go back to a higher risk level
      if( balances[i] == 0.0 ) continue;
      if( AccountBalance() >= balances[i] ) break;
    }
    switch(i)
    {
      case 0: risk = RiskPercent; break;
      case 1: risk = RiskPercent/2; break;
      case 2: risk = RiskPercent/4; break;
      default: risk = RiskPercent/4; break;
    }
    Print("Next trade risk will be at ", risk,"%");
    return;
  }  
}
    
      
// this function returns the last three levels of the swing zz indicator
bool CheckSwingValues()
{
  double val;
  double one, two, three;
  int i, j;
  
  i = -1; // set i to -1 because we want it to be zero on the first iteration
          // of the while loop, and we increment at the start of the loop.
  while( j < 3 )
  {
    i++;
    
    // get the ith value of the swing zz indicator  
    val = iCustom(Symbol(),TimeFrame,"swing_ZZ_with_alert",4,0,i);
    
    // most values will be zero, and we ignore those
    if( val != 0 )
    {
      // store the first three non-zero values
      if( j == 0 )
      {
        one = val;
        j++;
        continue;
      }
      if( j == 1 )
      {
        two = val;
        j++;
        continue;
      }
      if( j == 2 )
      {
        three = val;
        j++;
        continue;
      }
    }
    
  } // end while loop
  
  // currentValue is the most recent value
  currentValue = one;
  
  // values two and three will be our buy and sell levels, but we dont
  // know which way round they will be, but we know that the higher level
  // will be buy and the lower level will be sell
  if( two > three )
  {
    buyPrice = two;
    sellPrice = three;
  }
  else
  {
    buyPrice = three;
    sellPrice = two;
  }
  
  // show levels on screen
  Comment("Current Value = ", currentValue, " Buy Price = ", buyPrice, " Sell Price = ", sellPrice);
  
  // if values have changed, return true
  if( (oldSellPrice != sellPrice) || (oldBuyPrice != buyPrice) )
  {
    oldBuyPrice = buyPrice;
    oldSellPrice = sellPrice;
    return(true);
  }
  else
  {
    return(false);
  }
}

void DeleteOldOrders()
{
  int i;
  
  // delete buy orders
  for( i = 0 ; i < NumberOrders ; i++ )
  {
    if( myBuyTickets[i] > 0 )
    {
      OrderSelect(myBuyTickets[i],SELECT_BY_TICKET,MODE_TRADES);
      if( OrderType() == OP_BUYSTOP )
      {
        OrderDelete(myBuyTickets[i]);
        myBuyTickets[i] = 0;
      }
    }
  }
  if( TrailingOrder == true )
  {
    if( myBuyTickets[NumberOrders] > 0 )
    {
      OrderSelect(myBuyTickets[NumberOrders],SELECT_BY_TICKET,MODE_TRADES);
      if( OrderType() == OP_BUYSTOP )
      {
        OrderDelete(myBuyTickets[NumberOrders]);
        myBuyTickets[NumberOrders] = 0;
      }
    }
  }
  
  // delete sell orders
  for( i = 0 ; i < NumberOrders ; i++ )
  {
    if( mySellTickets[i] > 0 )
    {
      OrderSelect(mySellTickets[i],SELECT_BY_TICKET,MODE_TRADES);
      if( OrderType() == OP_SELLSTOP )
      {
        OrderDelete(mySellTickets[i]);
        mySellTickets[i] = 0;
      }
    }
  }
  if( TrailingOrder == true )
  {
    if( mySellTickets[NumberOrders] > 0 )
    {
      OrderSelect(mySellTickets[NumberOrders],SELECT_BY_TICKET,MODE_TRADES);
      if( OrderType() == OP_SELLSTOP )
      {
        OrderDelete(mySellTickets[NumberOrders]);
        mySellTickets[NumberOrders] = 0;
      }
    }
  }
  
  // clear order ticket arrays just to make sure
  for( i = 0 ; i < 11 ; i++ )
  {
    myBuyTickets[i] = 0;
    mySellTickets[i] = 0;
  }
}

void PlaceStopOrders()
{
  double spread;
  double sl;
  double buyTakeProfits[10];
  double sellTakeProfits[10];
  double riskAmount, lots;
  int i;
  int orders;
  

  spread = MarketInfo(Symbol(),MODE_SPREAD);
  
  // calculate take profits
  for( i = 0; i < NumberOrders ; i++ )
  {
    buyTakeProfits[i] = buyPrice+((OrderOneTakeProfit+spread+(TakeProfitInterval*i))*Point);
  }
  for( i = 0; i < NumberOrders ; i++ )
  {
    sellTakeProfits[i] = sellPrice-((OrderOneTakeProfit+spread+(TakeProfitInterval*i))*Point);
  }
  
  // if we are using MM, use risk calculated by that, otherwise use max risk.
  if( UseMM == true )
  {
    riskAmount = AccountEquity()*(risk/100);
  }
  else
  {
    riskAmount = AccountEquity()*(RiskPercent/100);
  }
  lots = riskAmount/(StopLoss*PricePerPip());
  
  // divide lots by number of orders;
  if( TrailingOrder == true )
  {
    orders = NumberOrders + 1;
  }
  else
  {
    orders = NumberOrders;
  }
  lots = lots/orders;
  lots = NormalizeDouble(lots,2);
  
  if( Ask < buyPrice )
  {
    sl = buyPrice-((StopLoss-spread)*Point);
    for( i = 0 ; i < NumberOrders ; i++ )
    {
      myBuyTickets[i] = OrderSend(Symbol(),OP_BUYSTOP,lots,buyPrice,3,sl,buyTakeProfits[i],0,MagicNumber,0,Green);
    }
    if( TrailingOrder == true )
    {
      myBuyTickets[NumberOrders] = OrderSend(Symbol(),OP_BUYSTOP,lots,buyPrice,3,sl,0,0,MagicNumber,0,Green);
    }
  }
  else
  {
    Print("Cant place buy stop as price > buy price" );
  }
  if( Bid > sellPrice )
  {
    sl = sellPrice+((StopLoss-spread)*Point);
    for( i = 0 ; i < NumberOrders ; i++ )
    {
      mySellTickets[i] = OrderSend(Symbol(),OP_SELLSTOP,lots,sellPrice,3,sl,sellTakeProfits[i],0,MagicNumber,0,Green);
    }
    if( TrailingOrder == true )
    {
      mySellTickets[NumberOrders] = OrderSend(Symbol(),OP_SELLSTOP,lots,sellPrice,3,sl,0,0,MagicNumber,0,Green);
    }
  }
  else
  {
    Print("Cant place sell stop as price < sell price" );
  }
}

double PricePerPip()
{
  double tickvalue, ticksize, priceperpip;
  
  tickvalue = MarketInfo(Symbol(),MODE_TICKVALUE);
  ticksize = MarketInfo(Symbol(),MODE_TICKSIZE);
  
  priceperpip = (tickvalue/ticksize)*Point;
  
  return(priceperpip);
}

void CheckTradeForStopLoss(int ticket, double ts)
{

  OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES);
  // check for opened position, symbol & MagicNumber
  if(OrderType()==OP_SELL && OrderMagicNumber() == MagicNumber)  
  { 
    
    if (OrderStopLoss()==0)
    {
      OrderModify(OrderTicket(),OrderOpenPrice(),Ask+Point*ts,OrderTakeProfit(),0,Red);
    }    
        
    if((OrderOpenPrice()-Ask)>(Point*ts))
    {
      if(OrderStopLoss()>(Ask+Point*ts))
      {
        OrderModify(OrderTicket(),OrderOpenPrice(),Ask+Point*ts,OrderTakeProfit(),0,Red);
      }
    }
         
  }
  
  if(OrderType()==OP_BUY && OrderMagicNumber() == MagicNumber)  
  { 
    if (OrderStopLoss()==0)
    {
      OrderModify(OrderTicket(),OrderOpenPrice(),Bid-Point*ts,OrderTakeProfit(),0,Green);
    }
             
    if(Bid-OrderOpenPrice()>Point*ts)
    {
      if(OrderStopLoss()<Bid-Point*ts)
      {
        OrderModify(OrderTicket(),OrderOpenPrice(),Bid-Point*ts,OrderTakeProfit(),0,Green);
      }
    }
  }
  return(0);
     
}

void WriteDayAndTime(int fileHandle)
{
  int dow;
  int hh, mm;
  string hours, mins, myTime;
  
  dow = TimeDayOfWeek(TimeCurrent());
  hh = TimeHour(TimeCurrent());
  mm = TimeMinute(TimeCurrent());
  
  if( hh < 10 ) hours = StringConcatenate("0",hh);
  else hours = hh;
  
  if( mm < 10 ) mins = StringConcatenate("0",mm);
  else mins = mm;
  
  myTime = StringConcatenate(hours,":",mins);
  
  FileWrite(fileHandle,dow,myTime);
  
  return;
}
  
  
  
  